function CreateScalableUnitExplosion( unit, overKillRatio )
    if IsUnit(unit) then
        local explosionEntity = CreateUnitExplosionEntity( unit, overKillRatio )
        WaitSeconds( Random( 0, 0.5 ) )
        unit:PlayUnitSound('Destroyed')
        ForkThread( _CreateScalableUnitExplosion, explosionEntity )
    end
end

function CreateTimedStuctureUnitExplosion( obj )
    local numExplosions = math.floor( GetAverageBoundingXYZRadius(obj) * GetRandomInt(5,10) )
    local x,y,z = GetUnitSizes(obj)
    obj:ShakeCamera( 30, 1, 0, 0.45 * numExplosions )
    for i = 0, numExplosions do
        CreateDefaultHitExplosionOffset( obj, 1.0, unpack({GetRandomOffset(x,y,z,1.2)}))
        obj:PlayUnitSound('DeathExplosion')
        WaitSeconds( GetRandomFloat( 0.1, 0.3 ))
    end
end

-- this is the 3d explosion for mobile units

function CreateTimedStuctureUnitExplosion2( obj )
    local numExplosions = math.floor( GetAverageBoundingXYZRadius(obj) * GetRandomInt(2,12) )
    if numExplosions > 60 then 
    	numExplosions = 60
    end
    local x,y,z = GetUnitSizes(obj)
    if numExplosions == 50 then
    	obj:ShakeCamera( numExplosions * 0.7 , 1, 0, 0.45 * numExplosions )    
	end
    for i = 0, numExplosions do
        CreateDefaultHitExplosionOffset( obj, 2.0, unpack({GetRandomOffset(x , y - 0.5 , z , 1.2)}))
        obj:PlayUnitSound('DeathExplosion')
        WaitSeconds( GetRandomFloat( 0.0, 0.01 ))
    end
end

-- this is the 3d explosion for air units

function CreateTimedStuctureUnitExplosion3( obj )
    local numExplosions = math.floor( GetAverageBoundingXYZRadius(obj) * GetRandomInt(12,24) )
    if numExplosions > 90 then 
    	numExplosions = 90
    end
    local x,y,z = GetUnitSizes(obj)
    for i = 0, numExplosions do
        CreateDefaultHitExplosionOffset( obj, 2.0, unpack({GetRandomOffset(x , y - 0.5 , z , 1.2)}))
        obj:PlayUnitSound('DeathExplosion')
        WaitSeconds( GetRandomFloat( 0.01, 0.04 ))
    end
end
-------------- experiment

---- experiment    

function _CreateScalableUnitExplosion( obj )
    local army = obj.Spec.Army
    local scale = obj.Spec.BoundingXZRadius
    local layer = obj.Spec.Layer
    local BaseEffectTable = {}
    local EnvironmentalEffectTable = {}
    local EffectTable = {}
    local ShakeTimeModifier = 0
    local ShakeMaxMul = 1 

    # Determine effect table to use, based on unit bounding box scale
    #LOG(scale)
    local randomized = Random( 1, 6)
    if scale < 0.5 then   ## Small units
    	BaseEffectTable = EffectTemplate.ExplosionEffectsSml01
    elseif scale > 2.5 then ## Large units
        BaseEffectTable = EffectTemplate.ExplosionEffectsLrg01
        ShakeTimeModifier = 1.0
        ShakeMaxMul = 0.25
    else                  ## Medium units
    	if randomized > 3 then
        BaseEffectTable = EffectTemplate.ExplosionEffectsMed01
        else
        BaseEffectTable = EffectTemplate.ExplosionEffectsMed02
        end	
    end
    
    # Get Environmental effects for current layer
    EnvironmentalEffectTable = GetUnitEnvironmentalExplosionEffects( layer, scale )

    # Merge resulting tables to final explosion emitter list
    if table.getn( EnvironmentalEffectTable ) != 0 then
        EffectTable = EffectTemplate.TableCat( BaseEffectTable, EnvironmentalEffectTable )
    else
        EffectTable = BaseEffectTable
    end
    
    #LOG( 'ExplosionEntity ', repr(obj), '\nEffect Table ', repr(EffectTable), '\nPosition ', repr(obj:GetPosition()) )

    #---------------------------------------------------------------
    # Create Generic emitter effects
    CreateEffects( obj, army, EffectTable )
    # Create Light particle flash
    CreateFlash( obj, -1, scale, army )
    # Create scorch mark
    if layer == 'Land' then
        if scale > 1.2 then
            CreateScorchMarkDecal( obj, scale, army )
        else
            CreateScorchMarkSplat( obj, scale, army )
        end
    end

    # Create GenericDebris chunks
    CreateDebrisProjectiles( obj, obj.Spec.BoundingXYZRadius, obj.Spec.Dimensions )
    # Camera Shake  (.radius .maxshake .minshake .lifetime)
    obj:ShakeCamera( 30 * scale, scale * ShakeMaxMul, 0, 0.5 + ShakeTimeModifier )
    #---------------------------------------------------------------
    #WaitSeconds( 5 )
    #_CreateScalableUnitExplosion( obj )
    obj:Destroy()
end

#---------------------------------------------------------------
# CREATELIGHTPARTICLE FLASH
#---------------------------------------------------------------
function CreateFlash( obj, bone, scale, army )
    CreateLightParticle( obj, bone, army, GetRandomFloat(2,8) * scale, GetRandomFloat(1, 5), 'glow_03', 'ramp_flare_02' )
end

function CreateDestructionSparks( object, scale )
    local proj
    for i = 1, GetRandomInt(10, 50) do
        proj = object:CreateProjectile('/effects/entities/DestructionSpark01/DestructionSpark01_proj.bp', 0, 0, 0, nil, nil, nil)
        proj:SetBallisticAcceleration(GetRandomFloat(-2, -3)):SetCollision(false)
        CreateEmitterOnEntity(proj,proj:GetArmy(),'/effects/emitters/destruction_explosion_sparks_02_emit.bp'):ScaleEmitter(scale)
    end
end

function CreateFirePlume( object, scale )
    local proj
    for i = 1, GetRandomInt(4, 8) do
        proj = object:CreateProjectile('/effects/entities/DestructionFirePlume01/DestructionFirePlume01_proj.bp', 0, 0, 0, nil, nil, nil)
        proj:SetBallisticAcceleration(GetRandomFloat(-2, -3)):SetCollision(false)
        local emitter = CreateEmitterOnEntity(proj,proj:GetArmy(),'/effects/emitters/destruction_explosion_fire_plume_02_emit.bp')
        #emitter:ScaleEmitter(scale)

        local lifetime = GetRandomFloat( 24, 48 )
        emitter:SetEmitterParam('REPEATTIME',lifetime)
        emitter:SetEmitterParam('LIFETIME', lifetime)
    end
end
