do
local OldShield = Shield

Shield = Class(OldShield) {
    OnCreate = function(self,spec)
        self.Trash = TrashBag()
        self.Owner = spec.Owner
        self.FactionName = spec.FactionName
		self:SetSize(spec.Size)
		-------------------------------------------------------------------------------------
        self:SetMaxHealth(spec.ShieldMaxHealth)
        self:SetHealth(self,spec.ShieldMaxHealth)
		-------------------------------------------------------------------------------------
        -- Show our 'lifebar'
		self:UpdateLifeBar()
		-------------------------------------------------------------------------------------
        self:SetShieldRegenRate(spec.ShieldRegenRate)
        self:SetShieldRegenStartTime(spec.ShieldRegenStartTime)
        self:SetRechargeTime(spec.ShieldRechargeTime or 5, spec.ShieldEnergyDrainRechargeTime or 5)
        self:SetVerticalOffset(spec.ShieldVerticalOffset)
		-------------------------------------------------------------------------------------
        self:SetVizToFocusPlayer('Always')
        self:SetVizToEnemies('Intel')
        self:SetVizToAllies('Always')
        self:SetVizToNeutrals('Intel')
		-------------------------------------------------------------------------------------
        self:AttachBoneTo(-1,spec.Owner,-1)
		-------------------------------------------------------------------------------------
		-- This is the primary change to this function, instead of turning on, go to the idle state
        self:Idle()
    end,

	-- Creates the collision shape used for the detect incoming fire
	-- Only create it if it doesn't already exist, as it is a heavyweight object in the engine
	CreateCollisionShape = function(self)
		if not self.CollisionShapeDefined then
			self:SetCollisionShape( 'Sphere', 0, 0, 0, self.Size/2)
			self:SetParentOffset(Vector(0,self.ShieldVerticalOffset,0))
			self:SetDrawScale(self.Size)
			self.CollisionShapeDefined = true
		end
	end,

	-- Called when the shield is neither active, nor idle
	RemoveCollisionShape = function(self)
		self:SetCollisionShape('None')
		self.CollisionShapeDefined = false
	end,

	-- Creates the shield mesh, overridden to change the collision shape code
	CreateShieldMesh = function(self)
		self:CreateCollisionShape()

		self:SetMesh(self.ShieldMesh[self.FactionName].Mesh)
		self:SetParentOffset(Vector(0,self.ShieldVerticalOffset,0))
		self:SetDrawScale(self.Size)

		if self.MeshZ == nil then
			self.MeshZ = Entity { Owner = self.Owner }
			self.MeshZ:SetMesh(self.ShieldMesh[self.FactionName].MeshZ)
            Warp( self.MeshZ, self.Owner:GetPosition() )
			self.MeshZ:SetDrawScale(self.Size)
			self.MeshZ:AttachBoneTo(-1,self.Owner,-1)
			self.MeshZ:SetParentOffset(Vector(0,self.ShieldVerticalOffset,0))

			self.MeshZ:SetVizToFocusPlayer('Always')
			self.MeshZ:SetVizToEnemies('Intel')
			self.MeshZ:SetVizToAllies('Always')
			self.MeshZ:SetVizToNeutrals('Intel')
		end
    end,
	
	-- Overridden to remove the code that removed the collision shape
	RemoveShield = function(self)
		self:SetMesh('')
		if self.MeshZ != nil then
			self.MeshZ:Destroy()
			self.MeshZ = nil
		end

		self.Owner:OnShieldDisabled()
    end,

	-- Pretty much rewritten, remove all the regen code
    OnDamage =  function(self,instigator,amount,vector,type)
		if self:IsOn() then
	        local absorbed = self:OnGetDamageAbsorption(instigator,amount,type)
	        self:AdjustHealth(instigator, -absorbed)
			self:UpdateLifeBar()
	        --LOG('Shield Health: ' .. self:GetHealth())
			-------------------------------------------------------------------------------------
	        if self:GetHealth() <= 0 then
	            ChangeState(self, self.RechargeState)
	        else
	            ForkThread(self.CreateImpactEffect, self, vector)
	        end
		end
	end,

	-- Switch to the idle state if not under fire
	DeactivateShieldThread = function(self)
		WaitSeconds(15)
		self:Idle()
	end,

	-- Conveniance function for forking the deactivate thread
	ForkDeactivateThread = function(self)
		self.DeactivateThread = ForkThread(self.DeactivateShieldThread, self)
		self.Owner.Trash:Add(self.DeactivateThread)
	end,

	-- Conveniance function for killing the deactivate thread
	KillDeactivateThread = function(self)
		if self.DeactivateThread then
		   KillThread(self.DeactivateThread)
		   self.DeactivateThread = nil
		end
	end,

	-- Switch to the idle state
	Idle = function(self)
		ChangeState(self, self.IdleState)
	end,

	-- Activate the shield in time to block the incoming fire
	ActivateShield = function(self)
		self:KillDeactivateThread()
		if self:IsOn() then
			self:ForkDeactivateThread()
		elseif self.Owner:GetScriptBit('RULEUTC_ShieldToggle') then
			self:TurnOn()
		end
	end,

	-- Behaive normally if the shield is active, if not active, activate
	OnCollisionCheckWeapon = function(self,firingWeapon)
		--LOG('*DEBUG: Weapon Collision Detected')
		local collide = OldShield.OnCollisionCheckWeapon(self,firingWeapon)
		if collide then
			self:ActivateShield()
		end
		return collide and self:IsOn()
	end,

	-- Behaive normally if the shield is active, if not active, activate
	OnCollisionCheck = function(self,other)
		--LOG('*DEBUG: Collision Detected')
		local collide = OldShield.OnCollisionCheck(self,other)
		if collide then
			self:ActivateShield()
		end
		return collide and self:IsOn()
	end,

	-- Recharge the shield at the rate specified
	RechargeShield = function(self, rate)
		self.Owner:SetMaintenanceConsumptionActive()
		while self:GetHealth() < self:GetMaxHealth() do
			WaitSeconds(1)
			local fraction = self.Owner:GetResourceConsumed()
			self:SetHealth(self, math.min( self:GetMaxHealth(), self:GetHealth() + rate * fraction ) )
			self:UpdateLifeBar()
		end
 	end,

	-- This was being used everywhere, I modified it and made it a function
	UpdateLifeBar = function(self)
		if not self.Owner:IsUnitState('Building') and
			not self.Owner:IsUnitState('Enhancing') and
			not self.Owner:IsUnitState('Repairing') then
			self.Owner:SetWorkProgress( math.max(0.00001, self:GetHealth() / self:GetMaxHealth()))
		end
	end,

	-------------------------------------------------------------------------------------------------------------------
	--                                                States
	-------------------------------------------------------------------------------------------------------------------

	-- This is the active state when the shield is on and blocking enemy fire
	-- It has been modified to drain the shields strength when power runs out instead of just turning off
	OnState = State {
        Main = function(self)
			self:KillDeactivateThread()
			-------------------------------------------------------------------------------------
			-- If the unit is attached to a transport, make sure the shield goes to the off state
			-- so the shield isn't turned on while on a transport
			--if self.Owner:IsUnitState('Attached') then
			--	self:TurnOff()
			--end
			-------------------------------------------------------------------------------------
			self:ForkDeactivateThread()
			self:UpdateLifeBar()
            self.Owner:OnShieldEnabled()
			self:CreateShieldMesh()
			-------------------------------------------------------------------------------------
            WaitSeconds(0.5)
            local fraction = self.Owner:GetResourceConsumed()
            local on = true
            local test = false
			local rate = self:GetMaxHealth() / self.ShieldRechargeTime
            while self:GetHealth() > 0 do
                WaitTicks(1)
                fraction = self.Owner:GetResourceConsumed()
				-- Drains shield strength if not recieving a of the 100% energy required
				-- Rate of drain based on how much energy we are short of 100%
				self:SetHealth(self, math.max( 0, self:GetHealth() - rate * (1.0 - fraction) ) )
				self:UpdateLifeBar()
            end
			-------------------------------------------------------------------------------------
			-- Ok, the shield is down. Switch to the recharge state
            ChangeState(self, self.RechargeState)
        end,

        IsOn = function(self)
            return true
        end,
    },

	OffState = State {
        Main = function(self)
			self:KillDeactivateThread()
            self:RemoveShield()
			self:RemoveCollisionShape()
			--while off, charge the health of the shield
			if self.Owner:GetScriptBit('RULEUTC_ShieldToggle') then
				self:RechargeShield( self:GetMaxHealth() / self.ShieldRechargeTime )
			end
			self.Owner:SetMaintenanceConsumptionInactive()
        end,
    },

	-- This is the idle state for when the shield is on, but not active
	-- Any incoming weapons fire will activate the shield and put it in the 'on' state
	IdleState = State {
        Main = function(self)
			self:KillDeactivateThread()
            self:RemoveShield()
			-------------------------------------------------------------------------------------
			self:CreateCollisionShape()
			-------------------------------------------------------------------------------------
			self:RechargeShield( self:GetMaxHealth() / self.ShieldRechargeTime )
			self.Owner:SetMaintenanceConsumptionInactive()
       end,
    },

	-- This state draws energy to recharge the shield, but won't activate the shield to block incoming shots
	RechargeState = State {
        Main = function(self)
			self:KillDeactivateThread()
			self:RemoveShield()
			self:RemoveCollisionShape()
			--while off, charge the health of the shield
			self:RechargeShield( self:GetMaxHealth() / self.ShieldRechargeTime )
			self.Owner:SetMaintenanceConsumptionInactive()
			self:Idle()
        end
    },

	-- These state are identical now, just point them at the RechargeState
	DamageRechargeState = Shield.RechargeState,
    EnergyDrainRechargeState = Shield.RechargeState,
}

local OldUnitShield = UnitShield
UnitShield = Class(Shield){
    OnCreate = function(self,spec)
        self.Trash = TrashBag()
        self.Owner = spec.Owner
        self.FactionName = spec.FactionName
        self.CollisionSizeX = spec.CollisionSizeX or 1
		self.CollisionSizeY = spec.CollisionSizeY or 1
		self.CollisionSizeZ = spec.CollisionSizeZ or 1
		self.CollisionCenterX = spec.CollisionCenterX or 0
		self.CollisionCenterY = spec.CollisionCenterY or 0
		self.CollisionCenterZ = spec.CollisionCenterZ or 0
		self.OwnerShieldMesh = spec.OwnerShieldMesh or ''
		-------------------------------------------------------------------------------------
		self:SetSize(spec.Size)
		-------------------------------------------------------------------------------------
        self:SetMaxHealth(spec.ShieldMaxHealth)
        self:SetHealth(self,spec.ShieldMaxHealth)
		-------------------------------------------------------------------------------------
        -- Show our 'lifebar'
		self:UpdateLifeBar()
		-------------------------------------------------------------------------------------
		self:SetRechargeTime(spec.ShieldRechargeTime or 5, spec.ShieldEnergyDrainRechargeTime or 5)
        self:SetVerticalOffset(spec.ShieldVerticalOffset)
		-------------------------------------------------------------------------------------
        self:SetVizToFocusPlayer('Always')
        self:SetVizToEnemies('Intel')
        self:SetVizToAllies('Always')
        self:SetVizToNeutrals('Always')
		-------------------------------------------------------------------------------------
        self:AttachBoneTo(-1,spec.Owner,-1)
		-------------------------------------------------------------------------------------
        self:SetShieldRegenRate(spec.ShieldRegenRate)
        self:SetShieldRegenStartTime(spec.ShieldRegenStartTime)
		-------------------------------------------------------------------------------------
		-- This is the primary change to this function, instead of turning on, go to the idle state
        self:Idle()
    end,

	-- Creates the collision shape used for the idle state to detect incoming fire in time to activate the shield
	CreateCollisionShape = function(self)
		if not self.CollisionShapeDefined then
			self:SetCollisionShape( 'Box', self.CollisionCenterX, self.CollisionCenterY, self.CollisionCenterZ, self.CollisionSizeX, self.CollisionSizeY, self.CollisionSizeZ)
			self.CollisionShapeDefined = true
		end
	end,

	-- Modified the collisionshape code
	CreateShieldMesh = function(self)
		self:CreateCollisionShape()
		self.Owner:SetMesh(self.OwnerShieldMesh,true)
	end,

	-- Modified to remove the collisionshape code
	RemoveShield = function(self)
		self.Owner:SetMesh(self.Owner:GetBlueprint().Display.MeshBlueprint, true)
		self.Owner:OnShieldDisabled()
    end,

	-- These are the original UnitShield functions
	CreateImpactEffect = OldUnitShield.CreateImpactEffect,
    OnDestroy = OldUnitShield.OnDestroy,
}

-- AnitArtilleryShields are not used, but for completenesses sake I have included it
local OldAntiArtilleryShield = AntiArtilleryShield
AntiArtilleryShield = Class(Shield){
	-- Behaive normally if the shield is active, if not active, activate
	OnCollisionCheckWeapon = function(self,firingWeapon)
		--LOG('*DEBUG: Weapon Collision Detected')
		local collide = OldAntiArtilleryShield.OnCollisionCheckWeapon(self,firingWeapon)
		if collide then
			self:ActivateShield()
		end
		return collide and self:IsOn()
	end,

	-- Behaive normally if the shield is active, if not active, activate
	OnCollisionCheck = function(self,other)
		--LOG('*DEBUG: Collision Detected')
		local collide = OldAntiArtilleryShield.OnCollisionCheck(self,other)
		if collide then
			self:ActivateShield()
		end
		return collide and self:IsOn()
	end,
}
end