do
local OldUnit = Unit
Unit = Class(OldUnit) {

    vetEmitter = nil,

    RecloakCountdown = 0,         -- This is the public countdown timer for weapons to set when they fire and make a unit decloak.
    CloakManagerThread = 0,
    Stopped = true,
    CloakEffectEnabled = false,

    DamageMultiplier = 1,

    IsBeingTransported = false,

    IsTeleportBeacon = false, 
    TeleportBeaconDetectionRadius = 30, 
    TeleportBeaconActive = false,
    TeleportBeaconIsBeingTransported = false,
    TeleportRecharging = false,
    InBeaconProximityTesting = false,
    NaturalTeleporter = false,
    HasTeleportUpgrade = false,
    IsTransportingTeleportBeacon = false,
    FreeTeleportRange = 0,
    TeleportEnergyCost = 0,
    TeleportRechargeTime = 0,

    TookDamageRecently = false,
    UnitRegenRate = 0,
    UnitBeingRepairedRate = 0,
    UnitDoneBuilding = false,
    FreeRepairRate = 0,
    DamageSponge = false,
                            -- This is the global tweak value that can be used to raise or lower the
    FudgeFactor = 8,        -- amount of Free Repairs across the board.
                            -- 8 = 40dps for T1 engineers, 80 for T2 and 120 for T3

    DamageCredit = 0,    #Add a new field to track damage dealt
    KillCredit = 0,   #Kills calculated by damage
    DamageConsideredKill = 0,

    ##########################################################################
    ##   Startup Function
    ##      This is the place to fork all threads that *every* unit should fork after being built.
    ##      eg self:ForkThread(self.PingSonar,self)
    ##########################################################################
    OnStopBeingBuilt = function(self, builder, layer)
        OldUnit.OnStopBeingBuilt(self, builder, layer)
        -- This is the place to fork all threads that *every* unit should fork after being built.
        --self:ForkThread(self.PingSonar,self)
        local bp = self:GetBlueprint()
        if bp.IsTeleportBeacon then
            self.IsTeleportBeacon = bp.IsTeleportBeacon
        end
        if bp.Physics.NaturalTeleporter == true then
            self.NaturalTeleporter = true
        end
        if bp.Physics.TeleportRange then
            self.FreeTeleportRange = bp.Physics.TeleportRange
        end
        if bp.Economy.TeleportEnergyMod then
            self.TeleportEnergyCost = bp.Economy.BuildCostEnergy * bp.Economy.TeleportEnergyMod
        end
        if bp.Physics.TeleportRechargeTime then
            self.TeleportRechargeTime = bp.Physics.TeleportRechargeTime
        end
        self.EnergyUsageRates = self:GetBlueprint().Economy.EnergyUsageRates
        if self.EnergyUsageRates then
            self:SetupEnergyUsage()
        end
        self.UnitRegenRate = self:GetBlueprint().Defense.RegenRate
        self.UnitDoneBuilding = true
    end,

    ##########################################################################
    ##   SetSpeedMult Manager
    ##        This section is overriding the SetSpeedMult function to track and 
    ##        allow units to modify their base speed.
    ##########################################################################
    
    BaseSpeed = 0,
    SpeedAdjustmentPercent = 1,
    CurrentSpeedMultiplier = 1,

    AdjustBaseSpeed = function(self, NewSpeedPercent)
        #This takes the new speed and applies it to the unit
        self.SpeedAdjustmentPercent = NewSpeedPercent
        #LOG("Adjusting Base Speed to: ".. NewSpeedPercent)
        self:SetSpeedMult(self.CurrentSpeedMultiplier)
    end,

    SetSpeedMult = function(self, mult)
        local ActualSpeedPercentage = ((self.SpeedAdjustmentPercent) * mult)
        #LOG("Current Speed:" ..self.CurrentSpeedMultiplier .. "  New Speed: " .. ActualSpeedPercentage .. " SpeedMult: " .. mult)
        self.CurrentSpeedMultiplier = mult
        OldUnit.SetSpeedMult(self, ActualSpeedPercentage)
    end,

    ###################################################
    #     Update Production Values
    #           This takes into account the increasing production values
    #           for hyrdros and mexes
    ##################################################

    UpdateProductionValues = function(self)
        local bpEcon = self:GetBlueprint().Economy
        if not bpEcon then return end
        self:SetProductionPerSecondEnergy((self.EnergyYield or bpEcon.ProductionPerSecondEnergy or 0) * (self.EnergyProdAdjMod or 1))
        self:SetProductionPerSecondMass((self.MassYield or bpEcon.ProductionPerSecondMass or 0) * (self.MassProdAdjMod or 1))
    end,

    ##########################################################################################
    ##  ENGINEERING
    ##########################################################################################

    OnStartBuild = function(self, unitBeingBuilt, order)
        if EntityCategoryContains(categories.ENGINEER, self) then
            --LOG("|| Engineer: " .. self:GetBlueprint().Description)
            --LOG("|| " .. order)
            if order != 'Upgrade' then
                if EntityCategoryContains(categories.MOBILE, unitBeingBuilt) then
                    if EntityCategoryContains(categories.TECH2, unitBeingBuilt) then
                        self:SetBuildTimeMultiplier(1.333)
                    elseif EntityCategoryContains(categories.TECH3, unitBeingBuilt) then
                        self:SetBuildTimeMultiplier(2)
                    end
                end
                if order == 'Repair' and unitBeingBuilt.UnitDoneBuilding == true then
                    if not unitBeingBuilt:IsUnitState('Upgrading') then
                        if self.BuildRateOverride then
                            self.FreeRepairRate = self.BuildRateOverride
                        else
                            self.FreeRepairRate = self:GetBlueprint().Economy.BuildRate
                        end
                        unitBeingBuilt.UnitBeingRepairedRate = unitBeingBuilt.UnitBeingRepairedRate + (self.FreeRepairRate * self.FudgeFactor)
                        if unitBeingBuilt.TookDamageRecently == false then
                            --LOG("|| START: Setting Regen Rate to " .. unitBeingBuilt.UnitRegenRate + unitBeingBuilt.UnitBeingRepairedRate)
                            unitBeingBuilt:SetRegenRate(unitBeingBuilt.UnitRegenRate + unitBeingBuilt.UnitBeingRepairedRate)
                        end
                    end
                end
            end
        end
        OldUnit.OnStartBuild(self,unitBeingBuilt,order)
    end,

    OnStopBuild = function(self, unitBeingBuilt)
        if unitBeingBuilt and not unitBeingBuilt:IsDead() then
            if self.FreeRepairRate > 0 then
                unitBeingBuilt.UnitBeingRepairedRate = unitBeingBuilt.UnitBeingRepairedRate - (self.FreeRepairRate * self.FudgeFactor)
                if unitBeingBuilt.TookDamageRecently == false then
                    --LOG("|| DISENGAGING: Setting Regen Rate to " .. unitBeingBuilt.UnitRegenRate + unitBeingBuilt.UnitBeingRepairedRate)
                    unitBeingBuilt:SetRegenRate(unitBeingBuilt.UnitRegenRate + unitBeingBuilt.UnitBeingRepairedRate)
                end
                self.FreeRepairRate = 0
            end
            self:SetBuildTimeMultiplier(1)
        end
        OldUnit.OnStopBuild(self,unitBeingBuilt)
    end,


    ##########################################################################################
    ##  VETERANCY
    ##########################################################################################

    DoTakeDamage = function(self, instigator, amount, vector, damageType)
        local healthBeforeHit =  self:GetHealth()
        local damageCredited = amount
        if(healthBeforeHit < amount)  then
            damageCredited = healthBeforeHit
        end
        local hisArmy = false
        if instigator and IsUnit(instigator) and instigator:GetArmy() then
            hisArmy = instigator:GetArmy()
        end
--         if instigator and not IsUnit(instigator) then
--             LOG("*debug: This unit is not a unit!! O_O")
--         end
        local myArmy = self:GetArmy()
     #   LOG("*debug: myArmy" .. repr(myArmy))
     #   LOG("*debug: hisArmy" .. repr(hisArmy))
     #   LOG("*debug: instigator" .. repr(instigator))
     #   LOG("*debug: self" .. repr(self))

        if hisArmy and myArmy != hisArmy then
            if not EntityCategoryContains(categories.BENIGN, self) then
                instigator:IncrementDamageCredit(damageCredited)
            end
        end
        #do the actual damaging bit


        -- NON VETERANCY CODE FOLLOWS:


        OldUnit.DoTakeDamage(self,instigator,amount,vector,damageType)
        --if self.TookDamageRecently == false then
        if not self:IsDead() then
            if self.DamageSponge == false then
                if self.DisableRegenAfterDamageThread then
                    self.DisableRegenAfterDamageThread:Destroy()
                    self.DisableRegenAfterDamageThread = nil
                end
                self.DisableRegenAfterDamageThread = self:ForkThread(self.DisableRegenAfterDamage, self)     -- This bit causes regen to pause in a fight.
            end
        end
    end,

    DisableRegenAfterDamage = function(self)
        self.TookDamageRecently = true
        --LOG("|| Setting regen to zero")
        self:SetRegenRate(0)
        WaitSeconds(10)
        --LOG("|| After 10 sec, Setting regen to " .. self.UnitRegenRate + self.UnitBeingRepairedRate)
        if not self:IsDead() then
            self:SetRegenRate(self.UnitRegenRate + self.UnitBeingRepairedRate)
            self.TookDamageRecently = false
        end
    end, 

    IncrementDamageCredit = function(self, damageDealt)
        damageDealt = damageDealt + self.DamageCredit
        self.DamageCredit = damageDealt
        #LOG('Dealt this much damage so far:', damageDealt)
        self:CheckVeteranLevel()
    end,

    OnKilledUnit = function(self, unitKilled)
        --Not going to do anything with this anymore - veterancy is calculated by damage dealt
        --self:CheckVeteranLevel()
    end,

    CheckVeteranLevel = function(self)
        -- call the original function first to set up the rest of the game UI
        local OldLevel
        local unitbp = self:GetBlueprint()

        OldLevel = self.VeteranLevel

        if (self.DamageConsideredKill == 0) then
            if( unitbp.dps) then
                self.DamageConsideredKill = unitbp.dps
            else
                self.DamageConsideredKill = 200
            end
        end


        #Now that we have the damage required to calculate kills, lets do it
        while (self.DamageCredit > self.DamageConsideredKill) do
            self.DamageCredit = self.DamageCredit - self.DamageConsideredKill
            self.KillCredit = self.KillCredit + 1
        end

        #Sync to UI
        self.Sync.KillCredit = self.KillCredit

        local vetbp = unitbp.Veteran
        if not vetbp then
            vetbp = Game.VeteranDefault
        end

        local nextLvl = self.VeteranLevel + 1  #Get the next level

        local nextKills = vetbp[('Level' .. nextLvl)]
        while (self.KillCredit >= nextKills) do
            if self.VeteranLevel == table.getsize(vetbp) then
                return
            end
            self:SetVeteranLevel(nextLvl)
            nextLvl = nextLvl + 1
            nextKills  = vetbp[('Level' .. nextLvl)]
        end
    end,

    #Set the veteran level to the level specified
    SetVeteranLevel = function(self, level)
          OldUnit.SetVeteranLevel(self,level)
          self:PlayVeteranFx(level)
    end,

    #PlayVeteranFx is staying here for backward compatibility... (original Unit.lua function)   
    PlayVeteranFx = function(self, newLvl)   
        self:CreateVeteranEmitter()   
    end,   
    
    #If you want to hide & show this effect with a toggle, just call   
    #Create and Destroy...   
    CreateVeteranEmitter = function(self)   
        if self.vetEmitter != nil then   
            self.vetEmitter:Destroy()   
            self.vetEmitter = nil   
        end   
        local VetScale = 0
        if self.VeteranLevel == 1 then
            VetScale = 0.6
        elseif self.VeteranLevel == 2 then
            VetScale = 0.8
        elseif self.VeteranLevel == 3 then
            VetScale = 1.0
        elseif self.VeteranLevel == 4 then
            VetScale = 1.2
        elseif self.VeteranLevel == 5 then
            VetScale = 1.4
        end
        self.vetEmitter = CreateAttachedEmitter(self, -1, self:GetArmy(),'/mods/supremilatedatwmedia_104/effects/emitters/veterancy_emitter.bp'):ScaleEmitter((self:GetEffectsScale()) * VetScale)   
    end,   
    
    DestroyVeteranEmitter = function(self)   
        if self.vetEmitter != nil then   
            self.vetEmitter:Destroy()   
            self.vetEmitter = nil   
        end   
    end,   
    
    
    #Set the veteran level to the level specified   
    SetVeteranLevel = function(self, level)   
        OldUnit.SetVeteranLevel(self,level)   
        self:PlayVeteranFx(level)   
    end,   


    ##########################################################################################
    ##  CLOAKING
    ##########################################################################################

    CloakManager = function(self)
        local MainLoopDelay = 1      -- Recommend this not be any greater than 1.
        local JustDisabled = false
        local IntelBP = self:GetBlueprint().Intel
        self.bpDecloakRadius = IntelBP.DecloakRadius or 0
        self.bpDecloakRadiusWhileMoving = IntelBP.DecloakRadiusWhileMoving or 0
        self.bpDecloakWhileMoving = false
        if IntelBP.DecloakWhileMoving and IntelBP.DecloakWhileMoving == true then
            self.bpDecloakWhileMoving = true
        end
        self.bpCloakDisableAfterNoEnergyTimer = IntelBP.CloakDisableAfterNoEnergyTimer or 0
        while not self:IsDead() do
            if self:GetScriptBit('RULEUTC_CloakToggle') == false then
                if self.RecloakCountdown > 0 then                     -- If RecloakCountdown is above zero, remain decloaked until it is zero.
                    self:Decloak()
                    self.RecloakCountdown = self.RecloakCountdown - 1
                else
                    local DecloakRadius = 0
                    if self.Stopped then
                        DecloakRadius = self.bpDecloakRadius
                    else
                        DecloakRadius = self.bpDecloakRadiusWhileMoving
                    end
                    if self.bpDecloakWhileMoving == true then
                        if self.Stopped == false then
                            self:Decloak()
                            JustDisabled = true
                        end
                    end
                    if self:EnemyInSphere(DecloakRadius) then  -- Default is no decloak radius.
                        self:Decloak()
                    elseif self.UnitBeingBuilt then             -- Decloak when building
                        self:Decloak()
                    else
                        local aiBrain = self:GetAIBrain()
                        local EnergyStored = aiBrain:GetEconomyStored('ENERGY')
                        if EnergyStored < 10 and self.bpCloakDisableAfterNoEnergyTimer > MainLoopDelay then
                            self:Decloak()                                      -- Decloak if out of energy.
                        else
                            if JustDisabled == false then
                                if EnergyStored > 10 then
                                    self:Cloak()
                                end
                            end
                        end
                    end
                end
            end
            JustDisabled = false
            WaitSeconds(MainLoopDelay) 
        end
    end,

    Cloak = function(self)
        if not(self:IsIntelEnabled('Cloak')) then 
            self:EnableUnitIntel('Cloak')
            self:DestroyVeteranEmitter()
            if self.EnergySwitch then
                self.EnergySwitch.Cloak = true
                self:UpdateEnergyUsage()
            else
                self:SetMaintenanceConsumptionActive()
            end
        end
    end,

    Decloak = function(self)
        if self:IsIntelEnabled('Cloak') then 
            self:DisableUnitIntel('Cloak')
            self:CreateVeteranEmitter()
            if self.EnergySwitch then
                self.EnergySwitch.Cloak = false
                self:UpdateEnergyUsage()
            else
                self:SetMaintenanceConsumptionInactive()
            end
        end
    end,

    OnIntelEnabled = function(self)
      OldUnit.OnIntelEnabled(self)
        local bp = self:GetBlueprint()
        local cloaked = self:IsIntelEnabled('Cloak') 
      #LOG('*DEBUG: IsIntelEnalbed(Cloak): ', repr(cloaked))
        if cloaked and not self.CloakEffectEnabled then
            #LOG('*DEBUG: Cloak Enabled')
        self.CloakEffectEnabled = true
        self:SetMesh(bp.Display.CloakMeshBlueprint , true)
        end
    end,

    OnIntelDisabled = function(self)
      OldUnit.OnIntelDisabled(self)
        if not self:IsIntelEnabled('Cloak') and self.CloakEffectEnabled then
            self.CloakEffectEnabled = false
            #LOG('*DEBUG: Cloak Disabled')
        self:SetMesh(self:GetBlueprint().Display.MeshBlueprint, true)
        end
    end,



    IntelWatchThread = function(self)
        WaitSeconds(0.5)
        local energy = self:GetConsumptionPerSecondEnergy()
        local mass = self:GetConsumptionPerSecondMass()
        local fraction = self:GetResourceConsumed()
        if energy == 0 and mass == 0 then
            fraction = 1
        end
        while fraction == 1 do
            WaitSeconds(0.5)
            energy = self:GetConsumptionPerSecondEnergy()
            mass = self:GetConsumptionPerSecondMass()
            if energy == 0 and mass == 0 then
                fraction = 1
            else
                fraction = self:GetResourceConsumed()
            end
        end
        self:DisableUnitIntel()
    end,

    ##########################################################################################
    ##  TELEPORTING
    ##########################################################################################

    OnTeleportUnit = function(self, teleporter, location, orientation)
        local TeleportSuccessful = false
        if self.TeleportRecharging == false then
            local PhysicsBP = self:GetBlueprint().Physics
            local MyPosition = self:GetPosition() 
            if self:TeleportBeaconsInSphere(MyPosition) >= 1 then
                if self:TeleportBeaconsInSphere(location) >= 1 then
                    OldUnit.OnTeleportUnit(self, teleporter, location, orientation)
                    TeleportSuccessful = true
                end
            end
            if TeleportSuccessful == false and self.FreeTeleportRange > 0 then
                local Distance = VDist2(MyPosition[1],MyPosition[3],location[1],location[3])
                if Distance < self.FreeTeleportRange then
                    OldUnit.OnTeleportUnit(self, teleporter, location, orientation)
                    TeleportSuccessful = true
                end
            end
            if TeleportSuccessful == false and PhysicsBP and PhysicsBP.TeleportAnywhere then
                OldUnit.OnTeleportUnit(self, teleporter, location, orientation)
                TeleportSuccessful = true
            end
            if TeleportSuccessful == true then
                if self.TeleportRechargeTime > 0 then
                    self:ForkThread(self.TeleportRechargeTimer,self)
                else
                    if self.WhiteRabbitCountdown then
                        self.WhiteRabbitCountdown = self.WhiteRabbitCountdown + 20
                    end
                end
            end
        end
    end,

    TeleportBeaconsInSphere = function(self,pos) 
        local x1 = pos.x - self.TeleportBeaconDetectionRadius 
        local y1 = pos.y - self.TeleportBeaconDetectionRadius 
        local z1 = pos.z - self.TeleportBeaconDetectionRadius 
        local x2 = pos.x + self.TeleportBeaconDetectionRadius 
        local y2 = pos.y + self.TeleportBeaconDetectionRadius 
        local z2 = pos.z + self.TeleportBeaconDetectionRadius 
        local units = GetUnitsInRect( Rect(x1, z1, x2, z2) ) 
        
        if not units then 
            return false 
        end 
        
        local army = self:GetArmy() 
        local BeaconCount = 0
        for k,v in units do 
            if v.IsTeleportBeacon == true then
                if(army == v:GetArmy()) then 
                    local dist = self:Get2DDistance(pos,v:GetPosition()) 
                    if dist <= self.TeleportBeaconDetectionRadius then 
                        if v.TeleportBeaconActive == true then
                            BeaconCount = BeaconCount + 1
                        end
                    end 
                end 
            end
        end 
        return BeaconCount
    end, 

    TeleportRechargeTimer = function(self)
        self.TeleportRecharging = true
        WaitSeconds(1)
        self:ForkThread(self.CreateTeleportEffects,self)
        self:RemoveCommandCap('RULEUCC_Teleport')
        WaitSeconds(math.max(0,self.TeleportRechargeTime - 1))
        self:AddCommandCap('RULEUCC_Teleport')
        self.TeleportRecharging = false
        self:CleanupTeleportChargeEffects()
        if self.NaturalTeleporter == false then
            if self.BeaconProximityThread == nil then
                self.BeaconProximityThread = self:ForkThread(self.TestBeaconProximity,self)
            end
        end
    end,

    CreateTeleportEffects = function(self)
        local aiBrain = self:GetAIBrain()
        while not self:IsDead() and aiBrain:GetEconomyStored('ENERGY') < 10 do
            WaitSeconds(0.5)
        end
        local army = self:GetArmy()
        self.TeleportChargeBag = {}
        local Scale = self:GetEffectsScale()
        for k, v in EffectTemplate.GenericTeleportCharge01 do
            local fx = CreateAttachedEmitter(self, -1, army,v):ScaleEmitter(Scale)
            self.Trash:Add(fx)
            table.insert( self.TeleportChargeBag, fx)
        end
    end,

    -- The beacons start this thread when a non-natural teleporter enters the circle.
    -- The thread continues to check for the existence of TWO beacons in range, and manage the teleport button accordingly.
    -- Thread dies when unit is no longer in range of ANY beacons.  Then it is up to a new beacon to start it again.
    -- This thread also starts up when a non-natural teleporter's teleport recharge has completed, to see if 
    --   it is still in range of TWO beacons.
    TestBeaconProximity = function(self)
        self.InBeaconProximityTesting = true
        local TeleportEnabled = false
        local RequiredBeaconCount = 0
        if self.HasTeleportUpgrade == true then
            RequiredBeaconCount = 1
        else
            if EntityCategoryContains(categories.NAVAL,self) then
                if EntityCategoryContains(categories.EXPERIMENTAL,self) then
                    RequiredBeaconCount = 4
                elseif EntityCategoryContains(categories.TECH3,self) then
                    RequiredBeaconCount = 4
                elseif EntityCategoryContains(categories.TECH2,self) then
                    RequiredBeaconCount = 3
                elseif EntityCategoryContains(categories.TECH1,self) then
                    RequiredBeaconCount = 2
                end
            else
                if EntityCategoryContains(categories.EXPERIMENTAL,self) then
                    RequiredBeaconCount = 4
                elseif EntityCategoryContains(categories.SUBCOMMANDER,self) then
                    RequiredBeaconCount = 1
                elseif EntityCategoryContains(categories.COMMAND,self) then
                    RequiredBeaconCount = 4
                elseif EntityCategoryContains(categories.TECH3,self) then
                    RequiredBeaconCount = 3
                elseif EntityCategoryContains(categories.TECH2,self) then
                    RequiredBeaconCount = 2
                elseif EntityCategoryContains(categories.TECH1,self) then
                    RequiredBeaconCount = 2
                end
            end
        end
        while (not self:IsDead()) and self.InBeaconProximityTesting == true do
            if self.IsTransportingTeleportBeacon == false then
                if self.TeleportRecharging == false then
                    local MyPosition = self:GetPosition() 
                    if self:TeleportBeaconsInSphere(MyPosition) >= RequiredBeaconCount then
                        if TeleportEnabled == false then
                            self:AddCommandCap('RULEUCC_Teleport')
                            TeleportEnabled = true
                        end
                    else
                        if TeleportEnabled == true then
                            self:RemoveCommandCap('RULEUCC_Teleport')
                            TeleportEnabled = false
                        end
                        if self:TeleportBeaconsInSphere(MyPosition) == 0 then
                            self.InBeaconProximityTesting = false
                        end
                    end
                end
            else
                if TeleportEnabled == true then
                    self:RemoveCommandCap('RULEUCC_Teleport')
                    TeleportEnabled = false
                end
            end
            WaitSeconds(1)
        end
        self.BeaconProximityThread:Destroy()
        self.BeaconProximityThread = nil
    end,

    OnLayerChange = function(self, new, old)      -- Set speed changes for layer transitions
        local PhysicsBP = self:GetBlueprint().Physics
        OldUnit.OnLayerChange(self, new, old)

        if new == 'Sub' then
            if PhysicsBP.LayerMods.Sub.Speed then
                self:SetSpeedMult(PhysicsBP.LayerMods.Sub.Speed)
            end
            if PhysicsBP.LayerMods.Sub.Acceleration then
                self:SetAccMult(PhysicsBP.LayerMods.Sub.Acceleration)
            end
            if PhysicsBP.LayerMods.Sub.TurnRate then
                self:SetTurnMult(PhysicsBP.LayerMods.Sub.TurnRate)
            end
        elseif new == 'Water' then
            if PhysicsBP.LayerMods.Water.Speed then
                self:SetSpeedMult(PhysicsBP.LayerMods.Water.Speed)
            end
            if PhysicsBP.LayerMods.Water.Acceleration then
                self:SetAccMult(PhysicsBP.LayerMods.Water.Acceleration)
            end
            if PhysicsBP.LayerMods.Water.TurnRate then
                self:SetTurnMult(PhysicsBP.LayerMods.Water.TurnRate)
            end
        elseif new == 'Land' then
            if PhysicsBP.LayerMods.Land.Speed then
                self:SetSpeedMult(PhysicsBP.LayerMods.Land.Speed)
            end
            if PhysicsBP.LayerMods.Land.Acceleration then
                self:SetAccMult(PhysicsBP.LayerMods.Land.Acceleration)
            end
            if PhysicsBP.LayerMods.Land.TurnRate then
                self:SetTurnMult(PhysicsBP.LayerMods.Land.TurnRate)
            end
        end
    end, 

    -- This function exists here ONLY to allow for better control over teleport energy costs.
    -- (and to remove the call to PlayTeleportChargeEffects and CleanupTeleportChargeEffects
    InitiateTeleportThread = function(self, teleporter, location, orientation)
        local tbp = teleporter:GetBlueprint()
        local ubp = self:GetBlueprint()
        self.UnitBeingTeleported = self
        self:SetImmobile(true)
        self:PlayUnitSound('TeleportStart')
        self:PlayUnitAmbientSound('TeleportLoop')
        local bp = self:GetBlueprint().Economy
        local energyCost, time
        if bp then
            local mass = bp.BuildCostMass * (bp.TeleportMassMod or 0.01)
            local energy = self.TeleportEnergyCost
            energyCost = mass + energy
            time = energyCost * (bp.TeleportTimeMod or 0.01)
        end

        #LOG('*UNIT DEBUG: TELEPORTING, energyCost = ', repr(energyCost), ' time = ', repr(time))
        self.TeleportDrain = CreateEconomyEvent(self, energyCost or 100, 0, time or 5, self.UpdateTeleportProgress)

        # create teleport charge effect
        --self:PlayTeleportChargeEffects()

        WaitFor( self.TeleportDrain  ) # Perform fancy Teleportation FX here
        self.TeleportDrain = nil
        self:PlayTeleportOutEffects()
        --self:CleanupTeleportChargeEffects()

        WaitSeconds( 0.1 )

        #Teleport Sound
        #LOG('DRAIN')
        #self.PlayUnitSound ('TeleportDrain')
        if self.TeleportDrain then
            RemoveEconomyEvent(self, self.TeleportDrain )
            self.TeleportDrain = nil
        end

        self:SetWorkProgress(0.0)
        Warp(self, location, orientation)
        self:PlayTeleportInEffects()

        WaitSeconds( 0.1 ) # Perform cooldown Teleportation FX here
        #Landing Sound
        #LOG('DROP')
        self:StopUnitAmbientSound('TeleportLoop')
        self:PlayUnitSound('TeleportEnd')
        self:SetImmobile(false)
        self.UnitBeingTeleported = nil
        self.TeleportThread = nil
    end,



    ##########################################################################################
    ##  TRANSPORTS
    ##########################################################################################

    OnTransportAttach = function(self, attachBone, unit)
        self:PlayUnitSound('Load')
        self:MarkWeaponsOnTransport(unit, true)
        if not EntityCategoryContains(categories.PODSTAGINGPLATFORM, self) then
            self:RequestRefreshUI()
        end
        unit.IsBeingTransported = true
        if unit.IsTeleportBeacon then
            if unit.IsTeleportBeacon == true then
                unit.TeleportBeaconIsBeingTransported = true
            end
        end
        self:RecalculateTransportSpeed()
    end,

    OnTransportDetach = function(self, attachBone, unit)
        OldUnit.OnTransportDetach(self, attachBone, unit) 
        unit.IsBeingTransported = false
        if unit.IsTeleportBeacon then
            if unit.IsTeleportBeacon == true then
                unit.TeleportBeaconIsBeingTransported = false
            end
        end
        self:RecalculateTransportSpeed()
    end,

    RecalculateTransportSpeed = function(self)
        local SpeedMultiplier = 1
        local bp = self:GetBlueprint().Transport
        local cargo = self:GetCargo()
        self.IsTransportingTeleportBeacon = false
        if table.getn(cargo) > 0 then
            for k, v in cargo do
                local TransportClass = v:GetBlueprint().Transport.TransportClass or 1
                if TransportClass == 1 then
                    SpeedMultiplier = SpeedMultiplier - (bp.Class1LoadSpeedPenalty or 0)
                elseif TransportClass == 2 then
                    SpeedMultiplier = SpeedMultiplier - (bp.Class2LoadSpeedPenalty or 0)
                elseif TransportClass == 3 then
                    SpeedMultiplier = SpeedMultiplier - (bp.Class3LoadSpeedPenalty or 0)
                end
                if v.IsTeleportBeacon then
                    if v.IsTeleportBeacon == true then
                        self.IsTransportingTeleportBeacon = true
                    end
                end
            end
        end
        self:SetSpeedMult(SpeedMultiplier)
        --self:SetAccMult(SpeedMultiplier)  -- Probably leave these out.
        --self:SetTurnMult(SpeedMultiplier)
    end,

    DeathWeaponDamageThread = function(self,damageRadius, damage, damageType, damageFriendly, EdgeDamageRatio)
        if damageRadius and damage then
            local Run1Step = false
            if not EdgeDamageRatio then
                Run1Step = true
            else
                if EdgeDamageRatio >= 1 then
                    Run1Step = true
                end
            end
            if damageRadius < 2 then
                Run1Step = true
            end
            if Run1Step then
                DamageArea(self, self:GetPosition(), damageRadius, damage, damageType or 'Normal', damageFriendly or true)
            else
                local EdgeDamage = damage * EdgeDamageRatio
                local HalfWay = math.ceil(damageRadius / 2)
                local EdgeFloor = math.floor(damageRadius)
                local StepCount = EdgeFloor - HalfWay
                local DamageFalloff = damage - EdgeDamage
                local StepDamage = DamageFalloff / StepCount
                local ThisDamageRing = HalfWay
                for i=1,StepCount do 
                    DamageArea(self, self:GetPosition(), ThisDamageRing, StepDamage, damageType or 'Normal', damageFriendly or true)
                    ThisDamageRing = ThisDamageRing + 1
                end
                DamageArea(self, self:GetPosition(), damageRadius, EdgeDamage, damageType or 'Normal', damageFriendly or true)
            end
        end
    end,

    DoDeathWeapon = function(self)
        if self:IsBeingBuilt() then return end
        local bp = self:GetBlueprint()
        for k, v in bp.Weapon do
            if(v.Label == 'DeathWeapon') then
                if v.FireOnDeath == true then
                    self:SetWeaponEnabledByLabel('DeathWeapon', true)
                    self:GetWeaponByLabel('DeathWeapon'):Fire()
                else
                    self:ForkThread(self.DeathWeaponDamageThread, v.DamageRadius, v.Damage, v.DamageType, v.DamageFriendly, v.EdgeDamageRatio)
                end
                break
            end
        end
    end,

    SetupBuildBones = function(self)  -- This sets up the blueprint tag "BuildYawSpeed" so you can control the speed of construction aiming.
        if not self.BuildBones or
           not self.BuildBones.YawBone or
           not self.BuildBones.PitchBone or
           not self.BuildBones.AimBone then
           return
        end
        # Syntactical reference:
        #CreateBuilderArmController(unit,turretBone, [barrelBone], [aimBone])
        #BuilderArmManipulator:SetAimingArc(minHeading, maxHeading, headingMaxSlew, minPitch, maxPitch, pitchMaxSlew)
        self.BuildArmManipulator = CreateBuilderArmController(self, self.BuildBones.YawBone or 0 , self.BuildBones.PitchBone or 0, self.BuildBones.AimBone or 0)
        local bp = self:GetBlueprint().Economy
        local BuildYawSpeed = bp.BuildYawSpeed or 360
        self.BuildArmManipulator:SetAimingArc(-180, 180, BuildYawSpeed, -90, 90, 360)
        self.BuildArmManipulator:SetPrecedence(5)
        if self.BuildingOpenAnimManip and self.BuildArmManipulator then
            self.BuildArmManipulator:Disable()
        end
        self.Trash:Add(self.BuildArmManipulator)
    end,

    OnDamage = function(self, instigator, amount, vector, damageType)
        amount = amount * self.DamageMultiplier 
        OldUnit.OnDamage(self, instigator, amount, vector, damageType)
    end, 

    SetArmoredState = function(self) 
        local ArmorMultiplier = self:GetBlueprint().Defense.ArmoredStateMultiplier or 1
        self.DamageMultiplier = ArmorMultiplier
    end, 

    SetUnarmoredState = function(self) 
        self.DamageMultiplier = 1 
    end,

    OnMotionHorzEventChange = function( self, new, old ) 
        OldUnit.OnMotionHorzEventChange(self, new, old) 
        if old == 'Stopped' then 
            self.Stopped = false
            local IntelBP = self:GetBlueprint().Intel
            if self.bpDecloakRadiusWhileMoving then
                self:SetIntelRadius('Sonar', self.bpDecloakRadiusWhileMoving)
            end
        elseif new == 'Stopped' then
            self.Stopped = true
            local IntelBP = self:GetBlueprint().Intel
            if self.bpDecloakRadius and self.bpDecloakRadiusWhileMoving then
                self:SetIntelRadius('Sonar', self.bpDecloakRadius)
            end
        end
    end,

    OnAnimCollision = function(self, bone, x, y, z)
        OldUnit.OnAnimCollision(self, bone, x, y, z)
        local layer = self:GetCurrentLayer()
        local bpTable = self:GetBlueprint().Display.MovementEffects

        if bpTable[layer].Footfall then
            bpTable = bpTable[layer].Footfall
            if bpTable.TreeFriendlyDamage then
                local bpDamage = bpTable.TreeFriendlyDamage
                local brain = self:GetAIBrain()
                local units = brain:GetUnitsAroundPoint(categories.VISIBLETORECON,self:GetPosition(bone),bpDamage.Radius,'Enemy')
                for k,v in units do
                    Damage(self,v:GetPosition(),v,bpDamage.Amount,'Normal')
                end
            end
        end
    end,

    ##########################################################################################
    ## BUFFS 
    ## For buffs, we're replacing the buff system entirely, and rewriting them to be more flexible 
    ## and more able to hand incorrect or automatically generated unit buffs. 
    ########################################################################################## 
     
    AddBuff = function(self, buffTable, PosEntity) 
        local bt = buffTable.BuffType 
        local unitBP = self:GetBlueprint()
        --LOG("*debug: Buffing Unit " .. (unitBP.General.Name or unitBP.BlueprintId) .." " .. bt .. " by " ..  (buffTable.Value or 0))
        if not bt then
            error('*ERROR: Tried to add a unit buff in unit.lua but got no buff table.  Wierd.', 1)
            return
        end
        #When adding debuffs we have to make sure that we check for permissions
        ### This is GPG's (odd) way of applying 'attack' buffs to enemy units. 
        local allow = categories.ALLUNITS 
        if buffTable.TargetAllow then
            allow = ParseEntityCategory(buffTable.TargetAllow)
        end 
        local disallow 
        if buffTable.TargetDisallow then 
            disallow = ParseEntityCategory(buffTable.TargetDisallow) 
        end 
        if bt == 'STUN' then 
            if buffTable.Radius and buffTable.Radius > 0 then
                #if the radius is bigger than 0 then we will use the unit as the center of the stun blast
                #and collect all targets from that point 
                local targets = {}
                if PosEntity then 
                    targets = utilities.GetEnemyUnitsInSphere(self, PosEntity, buffTable.Radius) 
                else 
                    targets = utilities.GetEnemyUnitsInSphere(self, self:GetPosition(), buffTable.Radius) 
                end 
                if not targets then 
                    #LOG('*DEBUG: No targets in radius to buff') 
                    return 
                end 
                for k, v in targets do 
                    if EntityCategoryContains(allow, v) and (not disallow or not EntityCategoryContains(disallow, v)) then 
                        v:SetStunned(buffTable.Duration or 1)
                    end 
                end 
            else 
                #The buff will be applied to the unit only 
                if EntityCategoryContains(allow, self) and (not disallow or not EntityCategoryContains(disallow, self)) then 
                    self:SetStunned(buffTable.Duration or 1) 
                end 
            end
        elseif bt == 'MAXHEALTH' then 
            if (unitBP.Defense.MaxHealth) then
                --LOG("*debug: Buffing Unit " .. (unitBP.General.Name or unitBP.BlueprintId) .." Maxhealth by " ..  (self:GetMaxHealth() + (buffTable.Value or 0)) )
                self:SetMaxHealth(self:GetMaxHealth() + (buffTable.Value or 0)) 
            end
        elseif bt == 'HEALTH' then
            if (unitBP.Defense.Health) then
                --LOG("*debug: Buffing Unit " ..(unitBP.General.Name or unitBP.BlueprintId) .." Health by " ..  (self:GetHealth() + (buffTable.Value or 0)))
                self:SetHealth(self, self:GetHealth() + (buffTable.Value or 0))
            end
        elseif bt == 'SPEEDMULT' then 
            if(EntityCategoryContains(categories.MOBILE, self)) then
                --LOG("*debug: Buffing Unit " .. (unitBP.General.Name or unitBP.BlueprintId) .." Speed by " ..  (buffTable.Value or 0))
                --self:SetSpeedMult(buffTable.Value or 0)
                self:AdjustBaseSpeed(buffTable.Value or 1)
            end
        elseif bt == 'MAXFUEL' then
            if (unitBP.Air.CanFly) then
                --LOG("*debug: Buffing Unit " .. (unitBP.General.Name or unitBP.BlueprintId) .." Maxfuel by " ..  (buffTable.Value or 0))
                self:SetFuelUseTime(buffTable.Value or 0)
            end
        elseif bt == 'FUELRATIO' then
            if (unitBP.Air.CanFly) then
                --LOG("*debug: Buffing Unit " .. (unitBP.General.Name or unitBP.BlueprintId) .." Fuel Time by " ..  (buffTable.Value or 0))
                self:SetFuelRatio(buffTable.Value or 0)
            end 
        elseif bt == 'HEALTHREGENRATE' then
            if (unitBP.Defense.MaxHealth and unitBP.Defense.Health) then
                --LOG("*debug: Buffing Unit " .. (unitBP.General.Name or unitBP.BlueprintId) .." Regen by " ..  (buffTable.Value or 0))
                self.UnitRegenRate = (self.UnitRegenRate + buffTable.Value) or self.UnitRegenRate
                self:SetRegenRate(self.UnitRegenRate + self.UnitBeingRepairedRate)
            end
        end
    end, 
    
    AddWeaponBuff = function(self, buffTable, weapon) 
        local bt = buffTable.BuffType
        local weaponBp = weapon:GetBlueprint()
        --LOG("*debug: Pre-Buffing Weapon " .. bt .. " by " .. (buffTable.Value or 0))
        if not bt then
            error('*ERROR: Tried to add a weapon buff in unit.lua but got no buff table.  Wierd.', 1) 
            return
        end 
        if bt == 'RATEOFFIRE' then 
            if (weaponBp.RateOfFire) then
                --LOG("*debug: Buffing Weapon " .. bt .. " by " .. (buffTable.Value or 0))
                weapon:ChangeRateOfFire(buffTable.Value or 1)
            end
        elseif bt == 'TURRETYAWSPEED' then
            if (weaponBp.TurretYawSpeed) then
                --LOG("*debug: Buffing Weapon " .. bt .. " by " .. (buffTable.Value or 0))
                weapon:SetTurretYawSpeed(buffTable.Value or 0) 
            end
        elseif bt == 'TURRETPITCHSPEED' then
            if (weaponBp.TurretPitchSpeed) then
                --LOG("*debug: Buffing Weapon " .. bt .. " by " .. (buffTable.Value or 0))
                weapon:SetTurretPitchSpeed(buffTable.Value or 0)
            end
        elseif bt == 'DAMAGE' then
            if (weaponBp.Damage) then
                --LOG("*debug: Buffing Weapon " .. bt .. " by " .. (buffTable.Value or 0))
                weapon:AddDamageMod(buffTable.Value or 0)
            end 
        elseif bt == 'MAXRADIUS' then 
            if (weaponBp.MaxRadius) then
                -- LOG("*debug: Buffing Weapon " .. bt .. " by " .. (buffTable.Value or 0))
                weapon:ChangeMaxRadius(buffTable.Value or weapon:GetBlueprint().MaxRadius) 
            end
        elseif bt == 'FIRINGRANDOMNESS' then
            if (weaponBp.FiringRandomness) then
                --LOG("*debug: Buffing Weapon " .. bt .. " by " .. (buffTable.Value or 0))
                weapon:SetFiringRandomness(buffTable.Value or 0)
            end
        else 
            self:AddBuff(buffTable) 
        end 
    end,

    ##########################################################################################
    ##  TAKE DAMAGE FROM AEON ABSOLVER
    ##########################################################################################

    TakeAbsolverDamage = function(self)
        self.TakingAbsolverDamage = true
        if not self.AbsolverEffectManager then
            self.AbsolverEffectManager = self:ForkThread(self.ManageAbsolverEffect)
        end
        WaitSeconds(1)
        self.TakingAbsolverDamage = false
    end,
    ManageAbsolverEffect = function(self)
        local bp = self:GetBlueprint()
        local AbsolverEffect = CreateAttachedEmitter(self, -1, self:GetArmy(),'/mods/supremilatedafterthewarsc/effects/emitters/absolver_damage.bp'):ScaleEmitter(10)
        while self.TakingAbsolverDamage == true and (not self:IsDead()) do
            WaitSeconds(1)
        end
        AbsolverEffect:Destroy()
        self.AbsolverEffectManager = nil
    end,


    ##########################################################################################
    ##  SHIELDS
    ##########################################################################################

    EnableShield = function(self)
        self:SetScriptBit('RULEUTC_ShieldToggle', true)
        if self.MyShield then
            self.MyShield:Idle()
        end
    end,

    ##########################################################################################
    ##  ENERGY MANAGEMENT
    ##########################################################################################

    SetupEnergyUsage = function(self)
        self.EnergySwitch = {}

        self:SetScriptBit('RULEUTC_CloakToggle', true)
        self:SetScriptBit('RULEUTC_JammingToggle', true)
        self:SetScriptBit('RULEUTC_StealthToggle', true)
        self:SetScriptBit('RULEUTC_SpecialToggle', true)
        self:SetScriptBit('RULEUTC_GenericToggle', true)
        self:SetScriptBit('RULEUTC_IntelToggle', true)
        self:SetScriptBit('RULEUTC_ProductionToggle', true)
        self:SetScriptBit('RULEUTC_ShieldToggle', true)
        self.EnergySwitch.Cloak = false
        self.EnergySwitch.CounterIntel = false
        self.EnergySwitch.Custom1 = false
        self.EnergySwitch.Custom2 = false
        self.EnergySwitch.Generic = false
        self.EnergySwitch.Intel = false
        self.EnergySwitch.Production = false
        self.EnergySwitch.Shield = false
        self:UpdateEnergyUsage()

        -------------------------------------------------------------------------------------
--[[
        -- LOGS to track what things are set to
        if self.EnergySwitch.Cloak == true then
            LOG("|| Cloak On")
        else
            LOG("|| Cloak Off")
        end
        if self.EnergySwitch.CounterIntel == true then
            LOG("|| CounterIntel On")
        else
            LOG("|| CounterIntel Off")
        end
        if self.EnergySwitch.Custom1 == true then
            LOG("|| Custom1 On")
        else
            LOG("|| Custom1 Off")
        end
        if self.EnergySwitch.Custom2 == true then
            LOG("|| Custom2 On")
        else
            LOG("|| Custom2 Off")
        end
        if self.EnergySwitch.Generic == true then
            LOG("|| Generic On")
        else
            LOG("|| Generic Off")
        end
        if self.EnergySwitch.Intel == true then
            LOG("|| Intel On")
        else
            LOG("|| Intel Off")
        end
        if self.EnergySwitch.Production == true then
            LOG("|| Production On")
        else
            LOG("|| Production Off")
        end
        if self.EnergySwitch.Shield == true then
            LOG("|| Shield On")
        else
            LOG("|| Shield Off")
        end
--]]
        -------------------------------------------------------------------------------------
    end,

    UpdateEnergyUsage = function(self)
        local Sum = 0
        --LOG("|| Starting at 0")
        if self.EnergySwitch.Cloak == true then
            if self.Stopped == true then
                Sum = Sum + (self.EnergyUsageRates.Cloak or 0)
            else
                if self.EnergyUsageRates.CloakWhileMoving then
                    Sum = Sum + self.EnergyUsageRates.CloakWhileMoving
                else
                    Sum = Sum + (self.EnergyUsageRates.Cloak or 0)
                end
            end
            --LOG("|| * Adding " .. (self.EnergyUsageRates.Cloak or 0) .. " energy usage for Cloak")
        end
        if self.EnergySwitch.CounterIntel == true then
            Sum = Sum + (self.EnergyUsageRates.CounterIntel or 0)
            --LOG("|| * Adding " .. (self.EnergyUsageRates.CounterIntel or 0) .. " energy usage for CounterIntel")
        end
        if self.EnergySwitch.Custom1 == true then
            Sum = Sum + (self.EnergyUsageRates.Custom1 or 0)
            --LOG("|| * Adding " .. (self.EnergyUsageRates.Custom1 or 0) .. " energy usage for Custom1")
        end
        if self.EnergySwitch.Custom2 == true then
            Sum = Sum + (self.EnergyUsageRates.Custom2 or 0)
            --LOG("|| * Adding " .. (self.EnergyUsageRates.Custom2 or 0) .. " energy usage for Custom2")
        end
        if self.EnergySwitch.Generic == true then
            Sum = Sum + (self.EnergyUsageRates.Generic or 0)
            --LOG("|| * Adding " .. (self.EnergyUsageRates.Generic or 0) .. " energy usage for Generic")
        end
        if self.EnergySwitch.Intel == true then
            Sum = Sum + (self.EnergyUsageRates.Intel or 0)
            --LOG("|| * Adding " .. (self.EnergyUsageRates.Intel or 0) .. " energy usage for Intel")
        end
        if self.EnergySwitch.Production == true then
            Sum = Sum + (self.EnergyUsageRates.Production or 0)
            --LOG("|| * Adding " .. (self.EnergyUsageRates.Production or 0) .. " energy usage for Production")
        end
        if self.EnergySwitch.Shield == true then
            Sum = Sum + (self.EnergyUsageRates.Shield or 0)
            --LOG("|| * Adding " .. (self.EnergyUsageRates.Shield or 0) .. " energy usage for Shield")
        end
        self.EnergyMaintenanceConsumptionOverride = Sum
        --LOG("|| Updating energy usage to " .. Sum)
        self:SetMaintenanceConsumptionActive()
        self:UpdateConsumptionValues()
    end,

    OnScriptBitSet = function(self, bit)
        if self.EnergySwitch then
            --LOG("|| Setting bit " .. bit)
            if bit == 0 then # shield toggle
                self.EnergySwitch.Shield = true
                self:EnableShield()
            elseif bit == 1 then # weapon toggle
                # do something
            elseif bit == 2 then # counterintel toggle
                -- Stealth, Jam & Spoof can go together because no unit would have more than one.
                --LOG("|| Setting EnergySwitch.CounterIntel to false")
                self.EnergySwitch.CounterIntel = false
                --LOG("|| Disable RadarStealth")
                self:DisableUnitIntel('RadarStealth')
                --LOG("|| Disable RadarStealthField")
                self:DisableUnitIntel('RadarStealthField')
                --LOG("|| Disable SonarStealth")
                self:DisableUnitIntel('SonarStealth')
                --LOG("|| Disable SonarStealthField")
                self:DisableUnitIntel('SonarStealthField')
                --LOG("|| Disable Jammer")
                self:DisableUnitIntel('Jammer')
                --LOG("|| Disable Spoof")
                self:DisableUnitIntel('Spoof')
            elseif bit == 3 then # intel toggle
                self.EnergySwitch.Intel = false
                --LOG("|| Disable Sonar")
                self:DisableUnitIntel('Sonar')
                --LOG("|| Disable Radar")
                self:DisableUnitIntel('Radar')
                --LOG("|| Disable Omni")
                self:DisableUnitIntel('Omni')
            elseif bit == 4 then # production toggle
                self.EnergySwitch.Production = false
                self:OnProductionPaused()
            elseif bit == 5 then # stealth toggle
                self.EnergySwitch.Custom1 = false
                # FREE TOGGLE!!
            elseif bit == 6 then # generic pause toggle
                self:SetPaused(true)
            elseif bit == 7 then # special toggle
                self.EnergySwitch.Custom2 = true
                self:EnableSpecialToggle()
            elseif bit == 8 then # cloak toggle
                self.EnergySwitch.Cloak = false
                --LOG("|| Disable Cloak")
                self:DisableUnitIntel('Cloak')
                --LOG("|| Disable CloakField")
                self:DisableUnitIntel('CloakField')
            end
            local AllStopped = true
            for k,v in self.EnergySwitch do 
                if v == true then
                    AllStopped = false
                    break
                end
            end
            if AllStopped == true then
                self:StopUnitAmbientSound( 'ActiveLoop' )
            end
            self:UpdateEnergyUsage()
        else
            OldUnit.OnScriptBitSet(self,bit)
        end
    end,

    OnScriptBitClear = function(self, bit)
        if self.EnergySwitch then
            --LOG("|| Clearing bit " .. bit)
            if bit == 0 then # shield toggle
                self.EnergySwitch.Shield = false
                self:DisableShield()
            elseif bit == 1 then # weapon toggle
                # do something
            elseif bit == 2 then # counterintel toggle
                -- Stealth, Jam & Spoof can go together because no unit would have more than one.
                --LOG("|| Setting EnergySwitch.CounterIntel to true")
                self.EnergySwitch.CounterIntel = true
                --LOG("|| Enable RadarStealth")
                self:EnableUnitIntel('RadarStealth')
                --LOG("|| Enable RadarStealthField")
                self:EnableUnitIntel('RadarStealthField')
                --LOG("|| Enable SonarStealth")
                self:EnableUnitIntel('SonarStealth')
                --LOG("|| Enable SonarStealthField")
                self:EnableUnitIntel('SonarStealthField')
                --LOG("|| Enable Jammer")
                self:EnableUnitIntel('Jammer')
                --LOG("|| Enable Spoof")
                self:EnableUnitIntel('Spoof')
            elseif bit == 3 then # intel toggle
                self.EnergySwitch.Intel = true
                --LOG("|| Enable Sonar")
                self:EnableUnitIntel('Sonar')
                --LOG("|| Enable Radar")
                self:EnableUnitIntel('Radar')
                --LOG("|| Enable Omni")
                self:EnableUnitIntel('Omni')
            elseif bit == 4 then # production toggle
                self.EnergySwitch.Production = true
                self:OnProductionUnpaused()
            elseif bit == 5 then # stealth toggle
                self.EnergySwitch.Custom1 = true
                # FREE TOGGLE!!
            elseif bit == 6 then # generic pause toggle
                self:SetPaused(false)
            elseif bit == 7 then # special toggle
                self.EnergySwitch.Custom2 = false
                self:DisableSpecialToggle()
            elseif bit == 8 then # cloak toggle
                self.EnergySwitch.Cloak = true
                --LOG("|| Enable Cloak")
                self:EnableUnitIntel('Cloak')
                --LOG("|| Enable CloakField")
                self:EnableUnitIntel('CloakField')
            end
            local AllStopped = true
            for k,v in self.EnergySwitch do 
                if v == true then
                    AllStopped = false
                    break
                end
            end
            if AllStopped == false then
                self:PlayUnitAmbientSound( 'ActiveLoop' )
            end
            self:UpdateEnergyUsage()
        else
            OldUnit.OnScriptBitClear(self,bit)
        end
    end,

    ##########################################################################################
    ##  UTILITIES
    ##########################################################################################

    EnemyInSphere = function(self,radius) 
        pos = self:GetPosition() 
        local units = GetUnitsInRect(self:MakeAreaRectangle(pos,radius)) 
        if not units then 
            return false 
        end 
        local army = self:GetArmy() 
        for k,v in units do 
            if(army != v:GetArmy()) then 
                if not EntityCategoryContains(categories.BENIGN, v) then  -- This should take out decloak due to enemy wall sections.
                    local dist = VDist3(pos,v:GetPosition()) 
                    if dist <= radius then 
                        return true 
                    end 
                end 
            end 
        end 
    end, 

    MakeAreaRectangle = function(self,Position,Radius)
        local x1 = Position.x - Radius 
        local y1 = Position.y - Radius 
        local z1 = Position.z - Radius 
        local x2 = Position.x + Radius 
        local y2 = Position.y + Radius 
        local z2 = Position.z + Radius 
        return Rect(x1, z1, x2, z2)
    end, 
   
    GetNearestEnemyInSphere = function(self,radius) -- This function not used in mod yet.
        local pos = self:GetPosition() 
        local units = GetUnitsInRect(self:MakeAreaRectangle(pos,radius)) 
        local NearestDistance
        local NearestEnemy = {}
        if not units then 
            return {}
        end 
        local army = self:GetArmy() 
        for k,v in units do 
            if(army != v:GetArmy()) then 
                local dist = VDist3(pos,v:GetPosition()) 
                if NearestDistance then
                    if dist < NearestDistance then
                        NearestEnemy = v
                        NearestDistance = dist
                    end
                else
                    NearestEnemy = v
                    NearestDistance = dist
                end 
            end 
        end
        if NearestDistance <= radius then
            return NearestEnemy
        else
            return {}
        end
    end,

    GetEffectsScale = function(self)
        local bp = self:GetBlueprint()
        local Scale = 1
        if bp.Display.EffectScale then
            Scale = math.max(bp.SizeX,bp.SizeY,bp.SizeZ) * bp.Display.EffectScale
        else
            if bp.BlueprintId == "uas0401" then
                Scale = math.max(bp.SizeX,bp.SizeY,bp.SizeZ) * 0.65
            elseif EntityCategoryContains(categories.NAVAL, self) then
                Scale = math.max(bp.SizeX,bp.SizeY,bp.SizeZ) * 0.25
            else
                Scale = math.max(bp.SizeX,bp.SizeY,bp.SizeZ) * 0.75
            end
        end
        return Scale
    end,

    Get2DDistance = function(self,Pos1,Pos2)
        return VDist2(Pos1[1], Pos1[3], Pos2[1], Pos2[3])
    end,

} TypeClass = Unit
end
