do
local OldUnit = UAL0001 
UAL0001 = Class(OldUnit) { 

    PhasedParticleInhibitorEnergyUsage = 100,
    PPIEnabled = true,
    EnergyConsumptionThread = {},

    OnCreate = function(self)
        OldUnit.OnCreate(self)
        self:AddBuildRestriction( categories.AEON * ( categories.BUILTBYTIER1ENGINEER + categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER + categories.EXPERIMENTAL) )
        local cat = ParseEntityCategory('BUILTBYCOMMANDER AEON')
        self:RemoveBuildRestriction(cat)
    end,

    EnableSpecialToggle = function(self)
        OldUnit.EnableSpecialToggle(self)
        if self.PPIEnabled == true then
            self:SetConsumptionPerSecondEnergy(self.PhasedParticleInhibitorEnergyUsage)
            self:SetConsumptionActive(true)
            self:SetArmoredState()
            self:SetRegenRate(1)
            self:SetImmobile(true)
            self:PlayPPIEffects()
            self.EnergyConsumptionThread = self:ForkThread(self.MaintainPPIEnergy,self) 
--            self:SetAllWeaponsEnabled(false)
            self:SetWeaponEnabledByLabel('RightDisruptor', false)
        end
    end,
    DisableSpecialToggle = function(self)
        OldUnit.DisableSpecialToggle(self)
        self.EnergyConsumptionThread:Destroy()
        self:ForkThread(self.PPIRecharge,self) 
        self:SetUnarmoredState()
        self:SetRegenRate(10)
        self:UpdateConsumptionValues()
        self.PhasedParticleInhibitorEnergyUsage = 100,
        self:ForkThread(self.WakeupAfterPPI,self) 
    end,
    MaintainPPIEnergy = function(self)
        while not self:IsDead() do 
            WaitSeconds(5)
            self.PhasedParticleInhibitorEnergyUsage = self.PhasedParticleInhibitorEnergyUsage * 1.15
            self:SetConsumptionPerSecondEnergy(self.PhasedParticleInhibitorEnergyUsage)
            self:SetConsumptionActive(true)
            if self:GetResourceConsumed() < 1 then
                self:DisableSpecialToggle()
            end
        end
    end,
    PPIRecharge = function(self)
        self.PPIEnabled = false
        WaitSeconds(180)
        self.PPIEnabled = true
    end,
    WakeupAfterPPI = function(self)
        WaitSeconds(15)
        self:SetImmobile(false)
--        self:SetAllWeaponsEnabled(true)
        self:SetWeaponEnabledByLabel('RightDisruptor', true)
        self:CleanupTeleportChargeEffects()
    end,
    PlayPPIEffects = function(self)
        local army = self:GetArmy()
        local bp = self:GetBlueprint()
        self.TeleportChargeBag = {}
        for k, v in EffectTemplate.GenericTeleportCharge01 do
            local fx = CreateAttachedEmitter(self, -1, army,v):ScaleEmitter(math.sqrt(bp.SizeX + bp.SizeZ) * math.sqrt(math.sqrt(bp.SizeX / bp.SizeZ)))
            self.Trash:Add(fx)
            table.insert( self.TeleportChargeBag, fx)
        end
    end,
    CreateEnhancement = function(self, enh)
        AWalkingLandUnit.CreateEnhancement(self, enh)
        local bp = self:GetBlueprint().Enhancements[enh]
        if enh == 'PPI' then
            self:AddToggleCap('RULEUTC_SpecialToggle')
        elseif enh == 'PPIRemove' then
            self:RemoveToggleCap('RULEUTC_SpecialToggle')
            self:DestroyShield()
        elseif enh == 'ShieldHeavy' then
            self:AddToggleCap('RULEUTC_ShieldToggle')
            self:SetEnergyMaintenanceConsumptionOverride(bp.MaintenanceConsumptionPerSecondEnergy or 0)
            self:CreatePersonalShield(bp)
        elseif enh =='Nanolathe' then
            local cat = ParseEntityCategory('BUILTBYTIER1ENGINEER AEON')
            self:RemoveBuildRestriction(cat)
            self:SetBuildRateOverride(bp.NewBuildRate)
        elseif enh =='NanolatheRemove' then
            self:SetBuildRateOverride(nil)
            self:RestoreBuildRestrictions()
            self:AddBuildRestriction( categories.AEON * ( categories.BUILTBYTIER1ENGINEER + categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER + categories.EXPERIMENTAL) )
            local cat = ParseEntityCategory('BUILTBYCOMMANDER AEON')
            self:RemoveBuildRestriction(cat)
        elseif enh =='AdvancedEngineeringRemove' then
            self:SetBuildRateOverride(nil)
            self:RestoreBuildRestrictions()
            self:AddBuildRestriction( categories.AEON * ( categories.BUILTBYTIER1ENGINEER + categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER + categories.EXPERIMENTAL) )
            local cat = ParseEntityCategory('BUILTBYCOMMANDER AEON')
            self:RemoveBuildRestriction(cat)
        elseif enh == 'EnhancedSensors' then
            self:SetIntelRadius('Vision', bp.NewVisionRadius)
            self:SetIntelRadius('Radar', bp.NewRadarRadius)
        elseif enh == 'EnhancedSensorsRemove' then
            local bpIntel = self:GetBlueprint().Intel
            self:SetIntelRadius('Vision', bpIntel.VisionRadius)
            self:SetIntelRadius('Radar', 0)
        elseif enh == 'HeatSink' then
            local bpDamage = self:GetBlueprint().Weapon[1].Damage
            local wep = self:GetWeaponByLabel('RightDisruptor')
            wep:AddDamageMod(bpDamage)
        elseif enh == 'HeatSinkRemove' then
            local bpDamage = self:GetBlueprint().Weapon[1].Damage
            local wep = self:GetWeaponByLabel('RightDisruptor')
            wep:AddDamageMod(-bpDamage)
        elseif enh == 'WeaponEnhanceLevel2' then
            local bpDamage = self:GetBlueprint().Weapon[1].Damage
            local wep = self:GetWeaponByLabel('RightDisruptor')
            wep:AddDamageMod(bpDamage * 2)
        elseif enh == 'WeaponEnhanceLevel2Remove' then
            local bpDamage = self:GetBlueprint().Weapon[1].Damage
            local wep = self:GetWeaponByLabel('RightDisruptor')
            wep:AddDamageMod(-bpDamage * 3)
        elseif enh == 'WeaponEnhanceLevel3' then
            local bpDamage = self:GetBlueprint().Weapon[1].Damage
            local wep = self:GetWeaponByLabel('RightDisruptor')
            wep:AddDamageMod(bpDamage * 4)
        elseif enh == 'WeaponEnhanceLevel3Remove' then
            local bpDamage = self:GetBlueprint().Weapon[1].Damage
            local wep = self:GetWeaponByLabel('RightDisruptor')
            wep:AddDamageMod(-bpDamage * 7)
        elseif enh == 'WeaponEnhanceLevel4' then
            local bpDamage = self:GetBlueprint().Weapon[1].Damage
            local wep = self:GetWeaponByLabel('RightDisruptor')
            wep:AddDamageMod(bpDamage * 8)
        elseif enh == 'WeaponEnhanceLevel4Remove' then
            local bpDamage = self:GetBlueprint().Weapon[1].Damage
            local wep = self:GetWeaponByLabel('RightDisruptor')
            wep:AddDamageMod(-bpDamage * 15)
        elseif enh == 'AdvancedTeleport' or enh == 'AdvancedTeleport2' then
            self:RemoveToggleCap('RULEUTC_SpecialToggle')
            self:AddCommandCap('RULEUCC_Teleport')
            self.FreeTeleportRange = 35
            self.TeleportEnergyCost = 0
            self.TeleportRechargeTime = 0
            self:ForkThread(self.WhiteRabbitManager,self) 
        elseif enh == 'AdvancedTeleportRemove' or enh == 'AdvancedTeleport2Remove'then
            self:RemoveCommandCap('RULEUCC_Teleport')
            self.FreeTeleportRange = 0
            self.TeleportEnergyCost = 15000
            self.TeleportRechargeTime = 60
        elseif enh == 'Teleporter' then
            self.HasTeleportUpgrade = true
            self.TeleportEnergyCost = 3000
            self.TeleportRechargeTime = 15
            self.InBeaconProximityTesting = false
        elseif enh == 'TeleporterRemove' then
            self.HasTeleportUpgrade = false
            self.TeleportRechargeTime = 60
            self.TeleportEnergyCost = 15000
            self.InBeaconProximityTesting = false
            self:RemoveCommandCap('RULEUCC_Teleport')
        elseif enh =='T3Engineering' then
            local bp = self:GetBlueprint().Enhancements[enh]
            if not bp then return end
            local cat = ParseEntityCategory(bp.BuildableCategoryAdds)
            self:RemoveBuildRestriction(cat)
            self:SetBuildRateOverride(bp.NewBuildRate)
        elseif enh =='T3EngineeringRemove' then
            local bp = self:GetBlueprint().Economy.BuildRate
            if not bp then return end
            self:SetBuildRateOverride(nil)
            self:RestoreBuildRestrictions()
            self:AddBuildRestriction( categories.AEON * ( categories.BUILTBYTIER1ENGINEER + categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER + categories.EXPERIMENTAL) )
            local cat = ParseEntityCategory('BUILTBYCOMMANDER AEON')
            self:RemoveBuildRestriction(cat)
        elseif enh =='T4Engineering' then
            local bp = self:GetBlueprint().Enhancements[enh]
            if not bp then return end
            local cat = ParseEntityCategory(bp.BuildableCategoryAdds)
            self:RemoveBuildRestriction(cat)
            self:SetBuildRateOverride(bp.NewBuildRate)
        elseif enh =='T4EngineeringRemove' then
            local bp = self:GetBlueprint().Economy.BuildRate
            if not bp then return end
            self:SetBuildRateOverride(nil)
            self:RestoreBuildRestrictions()
            self:AddBuildRestriction( categories.AEON * ( categories.BUILTBYTIER1ENGINEER + categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER + categories.EXPERIMENTAL) )
            local cat = ParseEntityCategory('BUILTBYCOMMANDER AEON')
            self:RemoveBuildRestriction(cat)
        elseif enh == 'ResourceAllocation' then
            -- Do nothing.
        elseif enh == 'ResourceAllocationRemove' then
            -- Do nothing.
        else
            OldUnit.CreateEnhancement(self, enh)
        end
    end,
    WhiteRabbitCountdown = 0,
    WhiteRabbitManager = function(self)
        local Overload = false
        while not self:IsDead() do 
            WaitSeconds(1)
            if self.WhiteRabbitCountdown >= 60 and Overload == false then
                local army = self:GetArmy()
                self.TeleportChargeBag = {}
                local Scale = self:GetEffectsScale()
                for k, v in EffectTemplate.GenericTeleportCharge01 do
                    local fx = CreateAttachedEmitter(self, -1, army,v):ScaleEmitter(Scale)
                    self.Trash:Add(fx)
                    table.insert( self.TeleportChargeBag, fx)
                end
                self:RemoveCommandCap('RULEUCC_Teleport')
                Overload = true
            end
            if self.WhiteRabbitCountdown < 60 and Overload == true then
                self:AddCommandCap('RULEUCC_Teleport')
                self:CleanupTeleportChargeEffects()
                if self.NaturalTeleporter == false then
                    if self.BeaconProximityThread == nil then
                        self.BeaconProximityThread = self:ForkThread(self.TestBeaconProximity,self)
                    end
                end
                Overload = false
            end
            if self.WhiteRabbitCountdown > 0 then
                self.WhiteRabbitCountdown = self.WhiteRabbitCountdown - 1
            end
        end
    end,
    OnTeleportUnit = function(self, teleporter, location, orientation)
        local TeleportSuccessful = false
        if self.WhiteRabbitCountdown < 60 then
            OldUnit.OnTeleportUnit(self, teleporter, location, orientation)
        end
    end,
} 
TypeClass = UAL0001
end 
