do
local OldUnit = UEL0001 
local TWalkingLandUnit = import('/lua/terranunits.lua').TWalkingLandUnit
local EffectUtilities = import('/lua/EffectUtilities.lua')
UEL0001 = Class(OldUnit) { 

    OnCreate = function(self)
        OldUnit.OnCreate(self)
        self:AddBuildRestriction( categories.UEF * ( categories.BUILTBYTIER1ENGINEER + categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER + categories.EXPERIMENTAL) )
        local cat = ParseEntityCategory('BUILTBYCOMMANDER UEF')
        self:RemoveBuildRestriction(cat)
    end,

    CreateEnhancement = function(self, enh)
        TWalkingLandUnit.CreateEnhancement(self, enh)
        if enh =='ResourceAllocation' then
            -- Do nothing.
        elseif enh == 'ResourceAllocationRemove' then
            -- Do nothing.
        elseif enh == 'DamageStablization' then
            local bpRegenRate = self:GetBlueprint().Enhancements.DamageStablization.NewRegenRate 
            self.UnitRegenRate = self.UnitRegenRate + bpRegenRate
            if self.TookDamageRecently == false then
                self:SetRegenRate(self.UnitRegenRate + self.UnitBeingRepairedRate)
            end
        elseif enh == 'DamageStablizationRemove' then
            local bpRegenRate = self:GetBlueprint().Enhancements.DamageStablization.NewRegenRate 
            self.UnitRegenRate = self.UnitRegenRate - bpRegenRate
            if self.TookDamageRecently == false then
                self:SetRegenRate(self.UnitRegenRate + self.UnitBeingRepairedRate)
            end
        elseif enh =='AdvancedEngineeringRemove' then
            self:SetBuildRateOverride(nil)
            self:RestoreBuildRestrictions()
            self:AddBuildRestriction( categories.UEF * ( categories.BUILTBYTIER1ENGINEER + categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER + categories.EXPERIMENTAL) )
            local cat = ParseEntityCategory('BUILTBYCOMMANDER UEF')
            self:RemoveBuildRestriction(cat)
        elseif enh =='HeavyAntiMatterCannon' then
            local bpROF = self:GetBlueprint().Weapon[1].RateOfFire
            local wep = self:GetWeaponByLabel('RightZephyr')
            wep:ChangeRateOfFire(bpROF * 2)
        elseif enh =='HeavyAntiMatterCannonRemove' then
            local bpROF = self:GetBlueprint().Weapon[1].RateOfFire
            local wep = self:GetWeaponByLabel('RightZephyr')
            wep:ChangeRateOfFire(bpROF)
        elseif enh =='WeaponEnhanceLevel2' then
            local bpROF = self:GetBlueprint().Weapon[1].RateOfFire
            local wep = self:GetWeaponByLabel('RightZephyr')
            wep:ChangeRateOfFire(bpROF * 4)
        elseif enh =='WeaponEnhanceLevel2Remove' then
            local bpROF = self:GetBlueprint().Weapon[1].RateOfFire
            local wep = self:GetWeaponByLabel('RightZephyr')
            wep:ChangeRateOfFire(bpROF)
        elseif enh =='WeaponEnhanceLevel3' then
            local bpDamage = self:GetBlueprint().Weapon[1].Damage
            local wep = self:GetWeaponByLabel('RightZephyr')
            wep:AddDamageMod(bpDamage)
        elseif enh =='WeaponEnhanceLevel3Remove' then
            local bpROF = self:GetBlueprint().Weapon[1].RateOfFire
            local bpDamage = self:GetBlueprint().Weapon[1].Damage
            local wep = self:GetWeaponByLabel('RightZephyr')
            wep:ChangeRateOfFire(bpROF)
            wep:AddDamageMod(-bpDamage)
        elseif enh =='WeaponEnhanceLevel4' then
            local bpDamage = self:GetBlueprint().Weapon[1].Damage
            local wep = self:GetWeaponByLabel('RightZephyr')
            wep:AddDamageMod(bpDamage * 2)
        elseif enh =='WeaponEnhanceLevel4Remove' then
            local bpROF = self:GetBlueprint().Weapon[1].RateOfFire
            local wep = self:GetWeaponByLabel('RightZephyr')
            wep:ChangeRateOfFire(bpROF)
            wep:AddDamageMod(-bpDamage * 3)
        elseif enh =='HeavyArmor' then
            local bpMaxHealth = self:GetBlueprint().Defense.MaxHealth + 60000
            self:SetMaxHealth(bpMaxHealth)
            self:SetHealth(self,bpMaxHealth)
        elseif enh =='HeavyArmorRemove' then
            local bpMaxHealth = self:GetBlueprint().Defense.MaxHealth
            local CurrentHealth = self:GetHealth()
            self:SetMaxHealth(bpMaxHealth)
            self:SetHealth(self,math.min(CurrentHealth,bpMaxHealth))
        elseif enh == 'Teleporter' then
            self.HasTeleportUpgrade = true
            self.InBeaconProximityTesting = false
            self.TeleportRechargeTime = 15
            self.TeleportEnergyCost = 3000
        elseif enh == 'TeleporterRemove' then
            self.HasTeleportUpgrade = false
            self.InBeaconProximityTesting = false
            self:RemoveCommandCap('RULEUCC_Teleport')
            self.TeleportRechargeTime = 60
            self.TeleportEnergyCost = 15000
        elseif enh =='T3Engineering' then
            local bp = self:GetBlueprint().Enhancements[enh]
            if not bp then return end
            local cat = ParseEntityCategory(bp.BuildableCategoryAdds)
            self:RemoveBuildRestriction(cat)
            self:SetBuildRateOverride(bp.NewBuildRate)
        elseif enh =='T3EngineeringRemove' then
            local bp = self:GetBlueprint().Economy.BuildRate
            if not bp then return end
            self:SetBuildRateOverride(nil)
            self:RestoreBuildRestrictions()
            self:AddBuildRestriction( categories.UEF * ( categories.BUILTBYTIER1ENGINEER + categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER + categories.EXPERIMENTAL) )
            local cat = ParseEntityCategory('BUILTBYCOMMANDER UEF')
            self:RemoveBuildRestriction(cat)
        elseif enh =='T4Engineering' then
            local bp = self:GetBlueprint().Enhancements[enh]
            if not bp then return end
            local cat = ParseEntityCategory(bp.BuildableCategoryAdds)
            self:RemoveBuildRestriction(cat)
            self:SetBuildRateOverride(bp.NewBuildRate)
        elseif enh =='T4EngineeringRemove' then
            local bp = self:GetBlueprint().Economy.BuildRate
            if not bp then return end
            self:SetBuildRateOverride(nil)
            self:RestoreBuildRestrictions()
            self:AddBuildRestriction( categories.UEF * ( categories.BUILTBYTIER1ENGINEER + categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER + categories.EXPERIMENTAL) )
            local cat = ParseEntityCategory('BUILTBYCOMMANDER UEF')
            self:RemoveBuildRestriction(cat)
        else
            OldUnit.CreateEnhancement(self,enh)
            local bp = self:GetBlueprint().Enhancements[enh]
            if not bp then return end
            if enh == 'ShieldGeneratorField' then
                local bp = self:GetBlueprint().Enhancements[enh]
                self:DestroyShield()
                self:CreatePersonalShield(bp)
            elseif enh =='Nanolathe' then
                local cat = ParseEntityCategory('BUILTBYTIER1ENGINEER UEF')
                self:RemoveBuildRestriction(cat)
                self:SetBuildRateOverride(bp.NewBuildRate)
            elseif enh =='NanolatheRemove' then
                self:AddBuildRestriction( categories.UEF * ( categories.BUILTBYTIER1ENGINEER + categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER + categories.EXPERIMENTAL) )
                local cat = ParseEntityCategory('BUILTBYCOMMANDER UEF')
                self:RemoveBuildRestriction(cat)
                self:SetBuildRateOverride(nil)
            end
        end
    end, 
} 
TypeClass = UEL0001
end
