do

local OldUnit = URL0301
local function EnhanceThread(self, builder)
    local FactoryEnhancements = builder.EnhanceList or {}
    for k,v in FactoryEnhancements do
        local order = {
            TaskName = "EnhanceTask",
            Enhancement = v
        }
        IssueScript({self}, order)
    end
end
URL0301 = Class(OldUnit) {
    OnStopBeingBuilt = function(self, builder, layer)
        OldUnit.OnStopBeingBuilt(self, builder, layer)
        self:ForkThread(EnhanceThread, builder)
    end,
    Weapons = {
        DeathWeapon = Class(CIFCommanderDeathWeapon) {},
        RightDisintegrator = Class(CDFLaserDisintegratorWeapon) {
            OnCreate = function(self)
                CDFLaserDisintegratorWeapon.OnCreate(self)
                #Disable buff 
                self:DisableBuff('STUN')
            end,
            OnWeaponFired = function(self)
                local bp = self:GetBlueprint()
                if self.unit.HasNinjaUpgrade and self.unit.HasNinjaUpgrade == true then
                    if self.unit.RecloakCountdown < (bp.UpgradedRecloakAfterFiringDelay or 0) then
                        self.unit.RecloakCountdown = bp.UpgradedRecloakAfterFiringDelay or 0
                    end
                else
                    if self.unit.RecloakCountdown < (bp.RecloakAfterFiringDelay or 0) then
                        self.unit.RecloakCountdown = bp.RecloakAfterFiringDelay or 0
                    end
                end
                CDFLaserDisintegratorWeapon.OnWeaponFired(self)
            end,
        },
        NMissile = Class(CAAMissileNaniteWeapon) {
            OnWeaponFired = function(self)
                local bp = self:GetBlueprint()
                if self.unit.HasNinjaUpgrade and self.unit.HasNinjaUpgrade == true then
                    if self.unit.RecloakCountdown < (bp.UpgradedRecloakAfterFiringDelay or 0) then
                        self.unit.RecloakCountdown = bp.UpgradedRecloakAfterFiringDelay or 0
                    end
                else
                    if self.unit.RecloakCountdown < (bp.RecloakAfterFiringDelay or 0) then
                        self.unit.RecloakCountdown = bp.RecloakAfterFiringDelay or 0
                    end
                end
                CAAMissileNaniteWeapon.OnWeaponFired(self)
            end,
        },
    },
    CreateEnhancement = function(self, enh)
        CWalkingLandUnit.CreateEnhancement(self, enh)
        if enh =='ResourceAllocation' then
            -- Do nothing.
        elseif enh == 'ResourceAllocationRemove' then
            -- Do nothing.
        elseif enh == 'CloakingGenerator' then
            self:AddToggleCap('RULEUTC_CloakToggle')
            self:SetScriptBit('RULEUTC_CloakToggle', true)
            self:SetScriptBit('RULEUTC_CloakToggle', false)
            self:RequestRefreshUI()
            self.RecloakCountdown = 0
            self.CloakManagerThread = self:ForkThread(self.CloakManager)
            self.CloakDevicePresent = true
        elseif enh == 'CloakingGeneratorRemove' then
            self:SetScriptBit('RULEUTC_CloakToggle', true)
            self:RemoveToggleCap('RULEUTC_CloakToggle')
            if self.CloakManagerThread then
                self.CloakManagerThread:Destroy()
            end
            self.CloakDevicePresent = false
            self:SetScriptBit('RULEUTC_JammingToggle', true)
            self:DisableUnitIntel('RadarStealth')
            self:DisableUnitIntel('SonarStealth')
            self:RemoveToggleCap('RULEUTC_JammingToggle')
            self:RequestRefreshUI()
        elseif enh == 'StealthGenerator' then
            self:AddToggleCap('RULEUTC_JammingToggle')
            self:EnableUnitIntel('RadarStealth')
            self:SetScriptBit('RULEUTC_JammingToggle', true)
            self:SetScriptBit('RULEUTC_JammingToggle', false)
            self:RequestRefreshUI()
        elseif enh == 'StealthGeneratorRemove' then
            self:SetScriptBit('RULEUTC_JammingToggle', true)
            self:DisableUnitIntel('RadarStealth')
            self:RemoveToggleCap('RULEUTC_JammingToggle')
            self:RequestRefreshUI()
        elseif enh == 'NaniteMissileSystem' then
            self:ShowBone('AA_Gun', true)
            self:SetWeaponEnabledByLabel('NMissile', true)
        elseif enh == 'NaniteMissileSystemRemove' then
            self:HideBone('AA_Gun', true)
            self:SetWeaponEnabledByLabel('NMissile', false)
        elseif enh == 'SelfRepairSystem' then
            local bpRegenRate = self:GetBlueprint().Enhancements.SelfRepairSystem.NewRegenRate 
            self:SetRegenRate(bpRegenRate or 0)
        elseif enh == 'SelfRepairSystemRemove' then
            self:RevertRegenRate()
        elseif enh =='Switchback' then
            self:SetBuildRateOverride(bp.NewBuildRate)
        elseif enh == 'SwitchbackRemove' then
            self:SetBuildRateOverride(nil)
        elseif enh == 'FocusConvertor' then
            local wep = self:GetWeaponByLabel('RightDisintegrator')
            wep:AddDamageMod(self:GetBlueprint().Enhancements.FocusConvertor.NewDamageMod or 0)
        elseif enh == 'FocusConvertorRemove' then
            local wep = self:GetWeaponByLabel('RightDisintegrator')
            wep:AddDamageMod(0)
        elseif enh == 'EMPCharge' then
            local wep = self:GetWeaponByLabel('RightDisintegrator')
            wep:ReEnableBuff('STUN')        
        elseif enh == 'EMPChargeRemove' then
            local wep = self:GetWeaponByLabel('RightDisintegrator')
            wep:DisableBuff('STUN')        
        end
        self:SetMaxHealth(self:GetMaxHealth() + self:GetBlueprint().Enhancements[enh].ArmorAdjustment) 
        if self:GetBlueprint().Enhancements[enh].ArmorAdjustment > 0 then
            self:SetHealth(self,self:GetHealth() + self:GetBlueprint().Enhancements[enh].ArmorAdjustment)
        else
            if self:GetHealth() > self:GetMaxHealth() then
                self:SetHealth(self,self:GetMaxHealth())
            end
        end
    end,
    OnKilled = function(self, instigator, type, overkillRatio)
        CWalkingLandUnit.OnKilled(self, instigator, type, overkillRatio)
    end,
    OnMotionHorzEventChange = function( self, new, old )
        OldUnit.OnMotionHorzEventChange(self, new, old)
        if self.CloakDevicePresent == true then
            if old == 'Stopped' then
                self:UpdateEnergyUsage()
            end
            if new == 'Stopped' then
                self:UpdateEnergyUsage()
            end
        end
    end,
}   
    
TypeClass = URL0301

end
